#!/opt/edge/venv/bin/python3.12

import semver

from edge.webserver.sysconf.uimod.umod import Umod
from edge.webserver.sysconf.uimod import rootinfo
from edge.webserver.sysconf.uimod import unet
from edge.webserver.sysconf.uimod.utype import *

_TARGET_VER = '0.37.58'


def _update_ver(unit_info):
    """
    :type unit_info: dict
    :rtype: None
    """
    ui_ref = unit_info['ref']
    ui_mod = Umod.create(ui_ref)
    ui_conf = ui_mod.load_conf()

    source_ver = ui_conf['ro'].get('ver')
    if source_ver is not None:
        if semver.Version.parse(source_ver).compare(_TARGET_VER) >= 0:
            return

    ui_conf['ro']['ver'] = _TARGET_VER
    ui_mod.save_conf(ui_conf)


def _main():
    # Must be first
    root_info = rootinfo.decode_info().root
    unet.map_units_r(root_info, map_fn=_update_ver, utypes=[
        UTYPE_DET_BUS_LANE,
        UTYPE_LOCATION,
        UTYPE_LPR,
        UTYPE_DET_PARKING,
        UTYPE_DET_RAILROAD,
        UTYPE_DET_REDLIGHT,
        UTYPE_DET_SEATBELT,
        UTYPE_DET_SPOT_SPEED,
        UTYPE_DET_SPOT_SPEED_WAGA,
        UTYPE_TERMINAL,
        UTYPE_DET_TRANSIT
    ])


if __name__ == '__main__':
    _main()
