#!/opt/edge/venv/bin/python3.12

from edge.support import debinterface
import glob

from typing import List


INTERFACES_FILEPATH = '/etc/network/interfaces'
MANAGEMENT_MAC_FILEPATH = '/etc/network/management-mac'

CAM_STATIC_MASK = '255.255.255.0'
CAM_STATIC_IP4 = '192.168.%s4.1'


def _get_unused_adapters(management_mac: str) -> List[str]:
    iface_names: List[str] = list()

    iface_filepath_list = glob.glob('/sys/class/net/*/address')
    for iface_filepath in iface_filepath_list:
        iface_name = iface_filepath.split('/')[-2]

        # exclude loopback adapter
        if iface_name == 'lo':
            continue

        with open(iface_filepath, 'r') as text_file:
            iface_mac = text_file.readline().strip()

            # exclude management interface
            if iface_mac != management_mac:
                iface_names.append(iface_name)

    return iface_names


def _setup_cam_interfaces(interfaces_path: str, iface_names: List[str]) -> None:
    iface_index = 1
    iface_conf = debinterface.interfaces.Interfaces(interfaces_path=interfaces_path)
    for iface_name in iface_names:
        new_adapter = debinterface.interfaces.NetworkAdapter(iface_name)
        new_adapter.setAuto(True)
        new_adapter.setAddrFam('inet')
        new_adapter.setAddressSource('static')
        new_adapter.setAddress(CAM_STATIC_IP4 % iface_index)
        new_adapter.setNetmask(CAM_STATIC_MASK)

        adapter_options = new_adapter.export()
        adapter_options['mtu'] = '6000'

        iface_conf.addAdapter(adapter_options, 0)

        iface_index += 1

    iface_conf.writeInterfaces()


def _main():
    with open(MANAGEMENT_MAC_FILEPATH, 'r') as text_file:
        management_mac = text_file.readline().strip()

    iface_names: List[str] = _get_unused_adapters(management_mac)

    _setup_cam_interfaces(INTERFACES_FILEPATH, iface_names)


if __name__ == '__main__':
    _main()
