#!/opt/edge/venv/bin/python3.12

from edge import eventlog
from edge.support import cgi
from edge.webserver.sysconf.api_controller import ControllerApi


class _HandlerFactory:
    """
    PUT    /api/v2/sys/conf/controller   /<uiref>/reboot
    POST   /api/v2/sys/conf/controller   /<uiref>/units
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'PUT': [
                cls._controller__put_uiref_reboot
            ],
            'POST': [
                cls._controller__post_uiref_units
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _controller__put_uiref_reboot(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'reboot':
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            ctrl_uiref = request.path_info[0]
            ControllerApi.reboot(ctrl_uiref)
            return cgi.CgiResponse()

        return handler

    @classmethod
    def _controller__post_uiref_units(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'units':
            raise cgi.CgiError404

        request.assert_query_absent()

        def handler() -> cgi.CgiResponse:
            ctrl_uiref = request.path_info[0]
            factory_conf = request.body_jnode if request.body else dict()
            if factory_conf.get('uuid') is not None:
                eventlog.log_e('fail: bad-args')
                raise cgi.CgiError400

            ext_terms_info = ControllerApi.create_subunit(ctrl_uiref, factory_conf)
            return cgi.CgiResponse(body=ext_terms_info)

        return handler


if __name__ == '__main__':
    eventlog.open_log('cgi-controller', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
