#!/opt/edge/venv/bin/python3.12

from edge import eventlog
from edge.support import cgi
from edge.webserver.sysconf.api_mv_camera import MvCameraApi


class _HandlerFactory:
    """
    GET   /api/v2/sys/conf/mv-camera   /<uiref>
    PUT   /api/v2/sys/conf/mv-camera   /<uiref>
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'GET': [
                cls._mv_camera__get_uiref
            ],
            'PUT': [
                cls._mv_camera__put_uiref
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _mv_camera__get_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            ext_uiconf = MvCameraApi.load_unit(request.path_info[0])
            return cgi.CgiResponse(body=ext_uiconf)

        return handler

    @classmethod
    def _mv_camera__put_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler() -> cgi.CgiResponse:
            ext_uiconf = MvCameraApi.save_unit(request.path_info[0], request.body_jnode)
            return cgi.CgiResponse(body=ext_uiconf)

        return handler


if __name__ == '__main__':
    eventlog.open_log('cgi-mv-camera', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
