#!/opt/edge/venv/bin/python3.12

import os

from edge import eventlog
from edge.support import fs, cgi
from edge.types.exceptions import *
from edge.webserver.sysconf.types.defs import CTRL_UICONF_DIR, CTRL_UISCHEMA_DIR


class _HandlerFactory:
    """
    GET     /api/v2/schemas   /<file/path>
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'GET': [
                cls._schemas__get_file
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _schemas__get_file(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            uiconf_path = fs.join_path(CTRL_UICONF_DIR, *request.path_info)
            uischema_path = fs.join_path(CTRL_UISCHEMA_DIR, *request.path_info)
            if os.path.islink(uischema_path) and fs.path_exists(os.readlink(uischema_path)):
                pass

            elif fs.path_exists(uiconf_path):
                fs.ln_schema(uischema_path, uiconf_path)

            else:
                eventlog.log_d('fail: NotFoundError ({})'.format(uiconf_path))
                raise NotFoundError

            schema_jnode = fs.load_json3(uischema_path, default=None)
            if schema_jnode is None:
                eventlog.log_d('fail: NotFoundError')
                raise NotFoundError

            return cgi.CgiResponse(body=schema_jnode)

        return handler


if __name__ == '__main__':
    eventlog.open_log('cgi-schema', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
