import ctypes

from PIL import Image

from edge.types.image import ImRgba
from edge.image.types.contrast import ContrastClaheConf
from edge.types.shape.rect import RectF
from edge_improc.improc import Rect as ImprocRect, ocv_improve_contrast_grba


class ImProc:
    def __init__(self, conf: ContrastClaheConf):
        self.conf = conf

    def improve_contrast_rgba_inplace(self, dh_data: ImRgba) -> None:
        ocv_improve_contrast_grba(
            self.conf.clip_limit, self.conf.grid_rows, self.conf.grid_cols, None,
            dh_data.w, dh_data.h, dh_data.stride,
            ctypes.cast(dh_data.buffer, ctypes.c_void_p),
            ctypes.cast(dh_data.buffer, ctypes.c_void_p))

    def improve_contrast_rgba(self, source_im: Image.Image, target_dh_data: ImRgba) -> None:
        """
        It is assumed that the target image has the same size as the source image.
        """
        assert source_im.width == target_dh_data.w
        assert source_im.height == target_dh_data.h

        target_dh_data.with_image(lambda im: im.paste(source_im, (0, 0)))
        ocv_improve_contrast_grba(
            self.conf.clip_limit, self.conf.grid_rows, self.conf.grid_cols, None,
            source_im.width, source_im.height, target_dh_data.stride,
            ctypes.cast(target_dh_data.buffer, ctypes.c_void_p),
            ctypes.cast(target_dh_data.buffer, ctypes.c_void_p))

    def improve_contrast_rgba_roi(self, source_im: Image.Image, target_dh_data: ImRgba, roi: RectF) -> None:
        """
        It is assumed that the target image has the same size as the source image.
        """
        assert source_im.width == target_dh_data.w
        assert source_im.height == target_dh_data.h

        target_dh_data.with_image(lambda im: im.paste(source_im, (0, 0)))

        improc_roi = ImprocRect(int(roi.fl), int(roi.ft), int(roi.fw), int(roi.fh))
        ocv_improve_contrast_grba(
            self.conf.clip_limit, self.conf.grid_rows, self.conf.grid_cols, ctypes.byref(improc_roi),
            source_im.width, source_im.height, target_dh_data.stride,
            ctypes.cast(target_dh_data.buffer, ctypes.c_void_p),
            ctypes.cast(target_dh_data.buffer, ctypes.c_void_p))
