from typing import Dict,  Final, Optional

from edge.image.types.contrast import ContrastConf
from edge.image.types.footer import ImFooterConf
from edge.image.types.markings import MarkingsConf


class PlateImConf:
    def __init__(self):
        self.width, self.height = 134, 30  # ~ 5:1


class ThumbImConf:
    def __init__(self):
        # self.width, self.height = 220, 165  # 4:3
        self.width, self.height = 224, 224  # 1:1 for MMR


class VehicleImAnnoConf:
    def __init__(self, plate: bool = False):
        self.plate: Final[bool] = plate


class VehicleImConf:
    def __init__(self, width: int, footer: ImFooterConf, anno: VehicleImAnnoConf):
        self.wh_ratio: Final[float] = 4.0 / 3.0

        self.width: Final[int | float] = width
        self.height: Final[int | float] = width / self.wh_ratio  # 4:3

        self.footer: Final[ImFooterConf] = footer
        self.anno: Final[VehicleImAnnoConf] = anno


class SceneImAnnoConf:
    def __init__(self, track: bool = False, vehicle: bool = False, plate: bool = False):
        self.track: Final[bool] = track
        self.vehicle: Final[bool] = vehicle
        self.plate: Final[bool] = plate


class SceneImConf:
    def __init__(self, width: int, footer: ImFooterConf, anno: SceneImAnnoConf, markings: MarkingsConf):
        self.width: Final[int] = width
        self.footer: Final[ImFooterConf] = footer
        self.anno: Final[SceneImAnnoConf] = anno
        self.markings: Final[MarkingsConf] = markings


class LiveImConf:
    def __init__(self, footer: ImFooterConf):
        self.footer: Final[ImFooterConf] = footer


class EventMeasImBuilderConf:
    def __init__(self,
                 scene_im: SceneImConf,
                 scene_footer: Dict,
                 scene_contrast: ContrastConf,
                 vehicle_im: Optional[VehicleImConf],
                 vehicle_footer: Optional[Dict],
                 vehicle_contrast: ContrastConf):

        self.plate: Final[PlateImConf] = PlateImConf()
        self.thumb: Final[ThumbImConf] = ThumbImConf()

        self.vehicle: Final[VehicleImConf] = vehicle_im
        self.vehicle_footer: Final[Dict] = vehicle_footer
        self.vehicle_contrast = vehicle_contrast

        self.scene: Final[SceneImConf] = scene_im
        self.scene_footer: Final[Dict] = scene_footer
        self.scene_contrast = scene_contrast
