import datetime
from typing import Optional, Final


class ContrastScheduleConf:
    MODE_ALWAYS = 'always'
    MODE_NEVER = 'never'
    MODE_NIGHT = 'twilight'
    MODE_TIME = 'time'

    def __init__(self, mode: str,
                 time_from: Optional[datetime.time] = None,
                 time_to: Optional[datetime.time] = None):

        self.mode: Final[str] = mode
        self.time_from: Final[datetime.time | None] = time_from
        self.time_to: Final[datetime.time | None] = time_to


class ContrastClaheConf:
    def __init__(self, clip_limit: float = 5.0, grid_cols: int = 8, grid_rows: int = 8):
        self.clip_limit: Final[float] = clip_limit
        self.grid_cols: Final[int] = grid_cols
        self.grid_rows: Final[int] = grid_rows

    def __eq__(self, that: object) -> bool:
        if not isinstance(that, ContrastClaheConf):
            return False

        return self.clip_limit == that.clip_limit and \
            self.grid_cols == that.grid_cols and \
            self.grid_rows == that.grid_rows


class ContrastConf:
    def __init__(self,
                 shedule: ContrastScheduleConf,
                 clache: ContrastClaheConf):

        self.shedule: Final[ContrastScheduleConf] = shedule
        self.clache: Final[ContrastClaheConf] = clache
