from typing import Dict

from edge import eventlog
from edge.image.types.jnode.contrast import decode__contrast_conf
from edge.image.types.jnode.footer import decode__im_footer_conf
from edge.image.types.jnode.markings import decode__markings_conf
from edge.types.exceptions import JnodeError
from edge.image.types.builder import (SceneImAnnoConf, VehicleImConf, VehicleImAnnoConf,
                                      SceneImConf, EventMeasImBuilderConf)

from edge.webserver.sysconf.uiconf.controller import CtrlUiconf


def decode__vehicle_im_conf(profile_jnode: Dict) -> VehicleImConf:
    """
    :raise JnodeError:
    """
    # deserialize #/width
    image_w = profile_jnode['width']
    if not (type(image_w) == int and 320 <= image_w <= 800):
        eventlog.log_e_param('#/width')
        raise JnodeError

    # deserialize #/footer
    footer_jnode = profile_jnode.get('footer')
    if dict != type(footer_jnode):
        eventlog.log_e_param('#/footer')
        raise JnodeError

    footer_args = decode__im_footer_conf(footer_jnode)

    # deserialize annotations
    anno_jnode = profile_jnode.get('annotations')
    anno = decode__vehicle_im_anno_conf(anno_jnode) if anno_jnode else VehicleImAnnoConf()

    return VehicleImConf(image_w, footer_args, anno)


def decode__vehicle_im_anno_conf(anno_jnode: Dict) -> VehicleImAnnoConf:
    """
    :raise JnodeError:
    """

    vehicle_jnode = anno_jnode.get('vehicle')
    if not vehicle_jnode:
        return VehicleImAnnoConf()

    plate = vehicle_jnode.get('plate', False)

    return VehicleImAnnoConf(plate)


def decode__scene_im_anno_conf(anno_jnode: Dict) -> SceneImAnnoConf:
    """
    :raise JnodeError:
    """

    vehicle_jnode = anno_jnode.get('vehicle')
    if not vehicle_jnode:
        return SceneImAnnoConf()

    track = vehicle_jnode.get('track', False)
    body = vehicle_jnode.get('body', False)
    plate = vehicle_jnode.get('plate', False)

    return SceneImAnnoConf(track, body, plate)


def decode__scene_im_conf(profile_jnode: Dict) -> SceneImConf:
    """
    :raise JnodeError:
    """
    # deserialize image-width
    image_w = profile_jnode['width']

    # deserialize footer-args
    footer_jnode = profile_jnode.get('footer')
    if not footer_jnode:
        eventlog.log_e('fail: param #/footer has invalid value')
        raise JnodeError

    footer = decode__im_footer_conf(footer_jnode)

    # deserialize annotations
    anno_jnode = profile_jnode.get('annotations')
    anno = decode__scene_im_anno_conf(anno_jnode) if anno_jnode else SceneImAnnoConf()

    markings_jnode = profile_jnode.get('markings') or dict()
    markings_conf = decode__markings_conf(markings_jnode)

    return SceneImConf(image_w, footer, anno, markings_conf)


def decode__im_builder_conf(jnode: Dict) -> EventMeasImBuilderConf:
    """
    :raise JnodeError:
    """
    #
    scene_jnode = jnode.get('scene')
    if not scene_jnode:
        eventlog.log_e_param('#/scene')
        raise JnodeError

    scene_im = decode__scene_im_conf(scene_jnode)

    scene_footer = scene_jnode.get('footer')
    if not scene_footer:
        eventlog.log_e_param('#/scene/footer')
        raise JnodeError

    scene_contrast = decode__contrast_conf(CtrlUiconf.get_contrast_scene())

    #
    vehicle_jnode = jnode.get('vehicle')
    if vehicle_jnode:
        vehicle_im = decode__vehicle_im_conf(vehicle_jnode)
        vehicle_footer = vehicle_jnode.get('footer')
        if not vehicle_footer:
            eventlog.log_e_param('#/vehicle/footer')
            raise JnodeError
    else:
        vehicle_im = None
        vehicle_footer = None

    vehicle_contrast = decode__contrast_conf(CtrlUiconf.get_contrast_vehicle())

    return EventMeasImBuilderConf(
        scene_im=scene_im,
        scene_footer=scene_footer,
        scene_contrast=scene_contrast,
        vehicle_im=vehicle_im,
        vehicle_footer=vehicle_footer,
        vehicle_contrast=vehicle_contrast)
