import datetime

from edge import eventlog
from edge.image.types.contrast import ContrastScheduleConf, ContrastClaheConf, ContrastConf
from edge.types.exceptions import JnodeError


def decode__contrast_shedule(shedule_jnode: list) -> ContrastScheduleConf:
    time_from = None
    time_to = None
    if len(shedule_jnode) == 0:
        mode = ContrastScheduleConf.MODE_NIGHT

    elif len(shedule_jnode) == 1:
        mode = shedule_jnode[0]
        if mode not in [ContrastScheduleConf.MODE_ALWAYS, ContrastScheduleConf.MODE_NEVER,
                        ContrastScheduleConf.MODE_NIGHT]:
            eventlog.log_e_param('#/mode')
            raise JnodeError

    elif len(shedule_jnode) == 2:
        mode = shedule_jnode[0]
        schedule_jnode = shedule_jnode[1]

        if mode != ContrastScheduleConf.MODE_TIME:
            eventlog.log_e_param('#/mode')
            raise JnodeError

        # deserialize #/time-from
        time_from_jnode = schedule_jnode.get('time-from')

        if not type(time_from_jnode) in [list, tuple] or len(time_from_jnode) != 2:
            eventlog.log_e_param('#/time-from')
            raise JnodeError

        hh, mm = time_from_jnode
        if not (int == type(hh) and 0 <= hh <= 23):
            eventlog.log_e_param('#/time-from/0')
            raise JnodeError

        if not (int == type(mm) and 0 <= mm <= 59):
            eventlog.log_e_param('#/time-from/1')
            raise JnodeError

        time_from = datetime.time(hh, mm)

        # deserialize #/time-to
        time_to_jnode = schedule_jnode['time-to']
        if not type(time_to_jnode) in [list, tuple] or len(time_to_jnode) != 2:
            eventlog.log_e_param('#/time-to')
            raise JnodeError

        hh, mm = time_to_jnode
        if not (int == type(hh) and 0 <= hh <= 23):
            eventlog.log_e_param('#/time-to/0')
            raise JnodeError

        if not (int == type(mm) and 0 <= mm <= 59):
            eventlog.log_e_param('#/time-to/1')
            raise JnodeError

        time_to = datetime.time(hh, mm)

    else:
        mode = ContrastScheduleConf.MODE_NEVER

    return ContrastScheduleConf(mode, time_from, time_to)


def decode__contrast_clahe(clache_jnode: dict) -> ContrastClaheConf:
    clip_limit = clache_jnode.get('clip-limit', 5.0)
    grid_size = clache_jnode.get('grid-size')

    if grid_size is not None:
        if type(grid_size) != list:
            eventlog.log_e_param('#/grid-size')
            raise JnodeError
        elif len(grid_size) != 2:
            eventlog.log_e_param('#/grid-size')
            raise JnodeError
        elif type(grid_size[0]) != int:
            eventlog.log_e_param('#/grid-size/0')
            raise JnodeError
        elif type(grid_size[1]) != int:
            eventlog.log_e_param('#/grid-size/1')
            raise JnodeError

        grid_rows = grid_size[0]
        grid_cols = grid_size[1]

    else:
        grid_cols = 8
        grid_rows = 8

    return ContrastClaheConf(clip_limit, grid_cols, grid_rows)


def decode__contrast_conf(args_jnode: dict) -> ContrastConf:
    """
    :raise JnodeError:
    """
    shedule_jnode = args_jnode.get('schedule', [])
    shedule = decode__contrast_shedule(shedule_jnode)

    clache_jnode = args_jnode.get('clahe', {})
    clache = decode__contrast_clahe(clache_jnode)
    return ContrastConf(shedule=shedule, clache=clache)
