from typing import Optional

from edge.types.timespan import TimeSpan


class Signal(list[TimeSpan]):
    def trim(self, pressision_us) -> None:
        for active_span in self:
            active_span.min_ts += pressision_us
            active_span.max_ts -= pressision_us

    def get_state(self, ts: int) -> bool:
        try:
            next(span for span in self if ts in span)
            return True

        except StopIteration:
            return False

    def get_min_ts(self, min_ts: Optional[int] = None, max_ts: Optional[int] = None) -> Optional[int]:
        """
        :param min_ts:
            The value None means no constraint (minus infinity).
        :param max_ts:
            The value None means no constraint (plus infinity).

        :returns:
            Min. timestamp of the active signal level in the interval (min_ts; max_ts].
        """
        if len(self) == 0:
            return None

        if min_ts is None:
            return self[0].min_ts

        if max_ts is None:
            # add 1 to include self last item in case of opened interval
            max_ts = self[-1].min_ts + 1

        for active_span in self:  # type: TimeSpan
            if min_ts < active_span.min_ts < max_ts:
                return active_span.min_ts

        return None

    def get_max_ts(self, min_ts: Optional[int] = None, max_ts: Optional[int] = None) -> Optional[int]:
        """
        :param self:
            The signal to analyze.
        :param min_ts:
            The value None means no constraint (minus infinity).
        :param max_ts:
            The value None means no constraint (plus infinity).

        :returns:
            Max. timestamp of the active signal level in the interval (min_ts; max_ts).
        """
        if len(self) == 0:
            return None

        if max_ts is None:
            return self[-1].max_ts

        if min_ts is None:
            min_ts = 0

        for active_span in reversed(self):
            if min_ts < active_span.max_ts < max_ts:
                return active_span.max_ts

        return None


def print_signal(name, signal: Signal) -> None:
    print(name)
    for active_span in signal:
        print(active_span.min_ts, '-', active_span.max_ts)
