from abc import abstractmethod
from typing import Final

from edge.services.types.service import IService
from edge.types.dir import Dir
from edge.types.track import LrTrackItem


class SpeedcamConf:
    def __init__(self, avg_plate_u: float | None, with_log: bool) -> None:
        self.avg_plate_u: Final[float | None] = avg_plate_u
        self.with_log: Final[bool] = with_log


class CamServiceConf:
    def __init__(self, intrinsics_path: str, extrinsics_path: str, speedcam2: SpeedcamConf) -> None:
        self.intrinsics_path: Final[str] = intrinsics_path
        self.extrinsics_path: Final[str] = extrinsics_path
        self.speedcam2: Final[SpeedcamConf] = speedcam2


class ICamService(IService):
    @abstractmethod
    def mk_image_point(self, scene_l: float, scene_u: float, scene_f: float) -> tuple[int, int]:
        """
        :raises ParamsError:
        """
        pass

    @abstractmethod
    def mk_world_point(self, image_x: int, image_y: int, world_t: float) -> tuple[float, float]:
        """
        :raises ParamsError:
        """
        pass

    @abstractmethod
    def assert_cam_speed_2(self,
                           vrp_text_utf8: bytes,
                           main_speed_kmh: int,
                           lr_items: list[LrTrackItem],
                           trackset_dir: Dir,
                           main_dir_u: bool = False) -> int:
        """
        :returns:
            Positive value means that the vehicle is moving in receeding direction relative to the control zone
            Negative value means that the vehicle is moving in approaching direction relative to the control zone

        :raises EstimateError:
            Failed to estimate the control speed.
        :raises ValidateError:
            Failed to validate the speed.
        """
        pass
