from typing import Dict, Optional

from edge.services.types.cam import CamServiceConf, SpeedcamConf
from edge.support import fs


def decode__cam_intrinsics_active(intrinsics_path: str) -> bool:
    """
    : camera_path:
        /etc/edge/setupstore/admin/actions/pipeline/0/mv-camera/0.json
        /etc/edge/setupstore/admin/actions/pipeline/0/rtsp-camera/0.json

    :raises JnodeError:
    """
    jnode: Optional[Dict] = fs.load_json3(intrinsics_path, default=None)
    if jnode is None:
        return False

    ro_jnode: Dict = jnode.get('ro') or dict()
    ro_intrinsic_jnode: Optional[Dict] = ro_jnode.get('intrinsic')

    rw_jnode: Dict = jnode.get('rw') or dict()
    rw_conf_jnode: Dict = rw_jnode.get('conf') or dict()
    rw_intrinsic_jnode: Optional[Dict] = rw_conf_jnode.get('intrinsic')

    intrinsic_jnode: Dict = rw_intrinsic_jnode or ro_intrinsic_jnode or dict()
    return intrinsic_jnode.get('active', True)


def decode__cam_service_conf(jnode: Optional[Dict]) -> Optional[CamServiceConf]:
    """
    : camera_path:
        main.conf#/services/cam
    :raises JnodeError:
    """
    if jnode is None:
        return None

    # /etc/edge/setupstore/admin/actions/pipeline/0/mv-camera/0.json
    # /etc/edge/setupstore/admin/actions/pipeline/0/rtsp-camera/0.json
    intrinsics_path: Optional[str] = jnode.get('intrinsics')
    if intrinsics_path is None:
        return None

    # /etc/edge/setupstore/admin/actions/pipeline/0/location/0.json
    extrinsics_path: Optional[str] = jnode.get('extrinsics')
    if extrinsics_path is None:
        return None
    elif not decode__cam_intrinsics_active(intrinsics_path):
        return None

    avg_plate_u: float | None = jnode.get('avg-plate-u')
    with_log: bool = jnode.get('log') or False

    speedcam: SpeedcamConf = SpeedcamConf(avg_plate_u=avg_plate_u, with_log=with_log)
    return CamServiceConf(intrinsics_path, extrinsics_path, speedcam)
