from typing import List, Dict

from edge import eventlog
from edge.services.types.lane import LaneServiceConf, LaneConf
from edge.types.exceptions import JnodeError
from edge.support.mask import decode__mask_region


def decode__lanes_conf(jnode: List[Dict], mask_w: int, mask_h: int) -> LaneServiceConf:
    """
    :param jnode:
        /etc/edge/speedcontrol.10/road.conf#/road/lanes
    :param mask_w:
        /etc/edge/speedcontrol.10/road.conf#/scene-wh/0
    :param mask_h:
        /etc/edge/speedcontrol.10/road.conf#/scene-wh/1

    :raise JnodeError:
    """
    if list != type(jnode):
        eventlog.log_e_param('#/mask/lanes')
        raise JnodeError

    lane_to_mask_list: List[LaneConf] = [(x['name'], decode__mask_region(x['mask'])) for x in jnode]
    return LaneServiceConf(lane_to_mask_list, mask_w, mask_h)
