from abc import abstractmethod
from datetime import datetime, tzinfo

from edge.services.types.service import IService


class TzServiceConf:
    def __init__(self, tz_info: tzinfo):
        self.tz_info = tz_info


class ITzService(IService):
    @abstractmethod
    def map_ts_sec_to_isostr(self, ts_sec: int | float) -> str:
        pass

    @abstractmethod
    def map_ts_usec_to_isostr(self, ts_usec: int | float) -> str:
        pass

    @abstractmethod
    def map_ts_sec_to_datetime_utc(self, ts_sec: int | float) -> datetime:
        pass

    @abstractmethod
    def map_ts_usec_to_datetime_utc(self, ts_usec: int | float) -> datetime:
        pass

    @abstractmethod
    def map_ts_sec_to_datetime_local(self, ts_sec: int | float) -> datetime:
        pass

    @abstractmethod
    def map_ts_usec_to_datetime_local(self, ts_usec: int | float) -> datetime:
        pass

    @abstractmethod
    def map_datetime_to_isostr(self, dt: datetime, with_usecs: bool = False) -> str:
        pass

    @abstractmethod
    def now_isostr(self) -> str:
        pass

    @abstractmethod
    def now_isostr_tssec(self) -> tuple[str, int]:
        pass
