import hglib

from typing import Any, Optional

from edge.support import fs

hglib.HGPATH = '/opt/edge/venv/bin/hg'


class ConfRepo:
    _HG_REPO_PATH = '/var/opt/edge/hg'
    _HG_CLONE_PATH = '/var/opt/edge/hg-clone'

    def __init__(self):
        self._repo: Optional[hglib.client] = None
        self._clone: Optional[hglib.client] = None

    def __enter__(self) -> (str, hglib.client):
        fs.silent_rm_r(self._HG_CLONE_PATH)
        self._repo = hglib.open(bytes(self._HG_REPO_PATH, 'ascii'))
        self._clone = hglib.clone(bytes(self._HG_REPO_PATH, 'ascii'), bytes(self._HG_CLONE_PATH, 'ascii')).open()
        return self._HG_CLONE_PATH, self._clone

    def __exit__(self, x_type: Optional[Any], x_self: Optional[Any], x_traceback: Optional[Any]) -> Optional[bool]:
        success = (x_type, x_self, x_traceback) == (None, None, None)
        if success:
            self._repo.pull(bytes(self._HG_CLONE_PATH, 'ascii'))

        self._repo.close()
        self._repo = None

        self._clone.close()
        self._clone = None
        fs.silent_rm_r(self._HG_CLONE_PATH)

        return success
