from edge.types.exceptions import ParamsError


class SpeedEstimateError(ParamsError):
    pass


class SpeedValidateError(ParamsError):
    pass


def map_error_rel_to_abs(main_speed: int, rel_error: float) -> float:
    return abs(main_speed) * rel_error


def get_speed_rel_error(main_speed: int, control_speed: int) -> float:
    """
    :param main_speed:
        Positive value means that the vehicle is moving in receeding direction relative to the control zone
        Negative value means that the vehicle is moving in approaching direction relative to the control zone
    :param control_speed:
        Positive value means that the vehicle is moving in receeding direction relative to the control zone
        Negative value means that the vehicle is moving in approaching direction relative to the control zone
    """
    rel_error: float = (main_speed - control_speed) / main_speed
    return abs(rel_error)


def get_speed_rel_error2(main_speed: int, control_speed: int) -> float:
    """
    :param main_speed:
        Value is always positive. The direction of vehicle movement is unknown.
        It is assumed that the direction of the main speed is the same as the direction of the control speed.
    :param control_speed:
        Positive value means that the vehicle is moving in receeding direction relative to the control zone
        Negative value means that the vehicle is moving in approaching direction relative to the control zone
    """
    rel_error: float = (main_speed - abs(control_speed)) / main_speed
    return abs(rel_error)


def get_speed_rel_error3(main_speed: int, control_speed: int, main_dir_u: bool = False) -> float:
    """
    :param main_dir_u:
            The sign main_speed_kmh does not indicate the direction of movement
            of the vehicle relative to the camera.
    """
    if main_dir_u:
        rel_error = get_speed_rel_error2(main_speed, control_speed)
    else:
        rel_error = get_speed_rel_error(main_speed, control_speed)
    return rel_error


def get_speed_abs_error(main_speed: int, control_speed: int, main_dir_u: bool = False) -> int:
    if main_dir_u:
        abs_error = main_speed - abs(control_speed)
    else:
        abs_error = main_speed - control_speed
    return abs(abs_error)


def assert_speed(main_speed_kmh: int, control_speed_kmh: int, main_dir_u: bool = False) -> int:
    # accept tracks with cam-speed difference less than 10% of main speed
    rel_error = get_speed_rel_error3(main_speed_kmh, control_speed_kmh, main_dir_u)
    if rel_error <= 0.10:
        return control_speed_kmh

    raise SpeedValidateError()


def map_speed_int(speed: float) -> int:
    return int(round(speed))


def map_speed_uint(speed: int | None) -> int | None:
    return speed and abs(speed)
