from typing import Callable, List

__all__ = [
    'AppError',
    'BufferSizeError',
    'BusyError',
    'CamServiceError',
    'GenericError',
    'ExternalError',
    'FsError',
    'InternalError',
    'JnodeError',
    'LicServiceError',
    'NotFoundError',
    'ParamsError',
    'ServiceError',
    'ToutError'
]


class GenericError(Exception):
    # http500
    pass

#
# ExternalError
#


class ExternalError(GenericError):
    # http400
    pass


class ParamsError(ExternalError):
    # http400
    pass


class NotFoundError(ExternalError):
    # http404
    pass


class BusyError(ExternalError):
    # http409
    pass


#
# InternalError
#


class InternalError(GenericError):
    # http500
    pass


class ToutError(InternalError):
    # http500
    pass


class FsError(InternalError):
    # http500
    pass


class ServiceError(InternalError):
    # http503
    pass


class CamServiceError(ServiceError):
    # http503
    pass


class LicServiceError(ServiceError):
    # http503
    pass


class AppError(RuntimeError):
    def __init__(self, log_entry: str):
        self._log = [log_entry]

    def extend(self, log_entry: str) -> None:
        self._log.append(log_entry)

    def foreach(self, fn: Callable[[str], None]) -> None:
        for entry in self._log[::-1]:  # reverse
            fn(entry)

    @property
    def log(self) -> List[str]:
        return self._log


class JnodeError(ValueError):
    pass


class BufferSizeError(BufferError):
    def __init__(self, min_size: int):
        self.min_size = min_size
