from typing import Dict

from edge.types.trackset.ra import RaTrack
from edge.types.track import RaTrackItem


def decode__ra_track_item(jnode: Dict) -> RaTrackItem:
    """
    :param jnode:
        TrTrack#/Ra/Trace/:x
    """
    scene_ts: int = jnode['scene-ts']
    scene_f_cm: int = jnode.get('F', 0)
    scene_l_cm: int = jnode.get('L', 0)

    frame_ts: int = jnode['Ts']
    frame_l_rel, frame_t_rel = jnode['Xy']

    speed_kmh: int = jnode['S']
    return RaTrackItem(frame_ts, frame_l_rel, frame_t_rel, scene_ts, scene_f_cm, scene_l_cm, speed_kmh)


def decode__ra_track(jnode: Dict) -> RaTrack:
    """
    :param jnode:
        TrTrack#/Ra
    """
    items = [decode__ra_track_item(item_jnode) for item_jnode in jnode['Trace']]

    speed_2_kmh = jnode.get('Speed2')
    speed_n_kmh = jnode['SpeedN']
    len_cm = jnode['Length']

    return RaTrack(items, speed_2_kmh, speed_n_kmh, len_cm)
