from typing import List, Self


class TimeSpan:
    def __init__(self, min_ts: int, max_ts: int):
        self.min_ts = min_ts
        self.max_ts = max_ts

    def __contains__(self, ts: int) -> bool:
        return self.min_ts <= ts <= self.max_ts

    @property
    def duration(self) -> int:
        return self.max_ts - self.min_ts

    def get_overlap(self, other: Self) -> Self | None:
        overlap_min_ts = max([self.min_ts, other.min_ts])
        overlap_max_ts = min([self.max_ts, other.max_ts])
        if overlap_min_ts <= overlap_max_ts:
            return TimeSpan(overlap_min_ts, overlap_max_ts)

    def test_overlaps(self, other: Self) -> bool:
        return self.get_overlap(other) is not None


def decode__timespan(jnode: List[int]) -> TimeSpan:
    """
    :param jnode:
        [min_ts, max_ts]
    :raise ValueError:
    """
    if len(jnode) != 2:
        raise ValueError

    return TimeSpan(*jnode)


def decode__timespans(jnodes: List[List[int]]) -> List[TimeSpan]:
    """
    :param jnodes:
        [[min_ts, max_ts], ...]
    :raise ValueError:
    """
    return [decode__timespan(jnode) for jnode in jnodes]
