from edge import eventlog
from edge.support import fs
from edge.types.exceptions import *
from edge.webserver.sysconf import edge_creator as creator
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.uimod import rootinfo
from edge.webserver.sysconf.uimod import unet
from edge.webserver.sysconf.uimod.umod import Umod
from edge.webserver.sysconf.uimod.utype import *


class WantedApi:
    @staticmethod
    def make_detector(term_uref: str, factory_conf: dict) -> tuple[str, dict]:
        if not unet.find_unit_r(rootinfo.decode_info().root, uref=term_uref):
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        unit_utype = factory_conf.get('type')
        unit_name = factory_conf.get('name')

        if unit_utype != UTYPE_DET_WANTED or unit_name is None:
            eventlog.log_d('fail: ParamsError')
            raise ParamsError

        unit_uref, root_uinfo = ph.RootInfoUiconf.add_term_detector(term_uref, unit_utype, unit_name)
        unit_uconf = ph.build_term_unit_uconf(term_uref, unit_utype, unit_uref.ref, factory_conf)
        unit_umod = Umod(unit_uref.pid, unit_uref.type, unit_uref.id)
        unit_umod.install(unit_uconf)

        unit_uconf = unit_umod.load_conf()
        unit_uconf['rw']['args']['name'] = unit_name

        urefs_l0_l1 = rootinfo.get_linked_urefs(unit_umod.ref)
        creator.generate(unit_umod.create_backup_dirpath(), urefs_l0_l1, {unit_umod.ref: unit_uconf})
        return unit_umod.ref, root_uinfo

    @staticmethod
    def drop_detector(uref: str) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_DET_WANTED)
        if unit_umod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        root_uinfo, urefs_l1 = rootinfo.drop_unit_r(unit_umod.ref)
        creator.generate(unit_umod.create_backup_dirpath(), urefs_l1, forced=True)
        unit_umod.uninstall()

        return {
            'info': [root_uinfo]
        }

    @staticmethod
    def load_detector(uref: str) -> dict:
        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_DET_WANTED)

        unit_uconf = unit_umod.load_conf(default=None)
        if unit_uconf is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        reply = {
            'type': UTYPE_DET_WANTED,
            'conf': unit_uconf['rw']['args']
        }

        reply.update(ph.get_detector_limits_stats(uref, scene=False, vehicle=False))
        return reply

    @staticmethod
    def save_detector(uref: str, new_uconf: dict) -> dict:
        new_uconf_conf = new_uconf.get('conf')

        unit_umod = Umod.create(uref)
        unit_umod.assert_type(UTYPE_DET_WANTED)

        if unit_umod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        fs.validate_json({'args': new_uconf_conf}, unit_umod.schema_path)

        unit_uconf = unit_umod.load_conf()
        if unit_uconf['rw']['args'] != new_uconf_conf:
            if new_uconf_conf['name'] != unit_uconf['rw']['args']['name']:
                rootinfo.set_unit_name(unit_umod.ref, new_uconf_conf['name'])

            unit_uconf['rw']['args'] = new_uconf_conf

            urefs_l0_l1 = rootinfo.get_linked_urefs(unit_umod.ref)
            creator.generate(unit_umod.create_backup_dirpath(), urefs_l0_l1, {unit_umod.ref: unit_uconf})

        reply = {
            'type': UTYPE_DET_WANTED,
            'conf': new_uconf_conf
        }
        reply.update(ph.get_detector_limits_stats(uref, scene=False, vehicle=False))
        return reply
