from edge.types.identity import Identity


def create_unit__leddar___leddar(unit_name: str, leddar_conf: dict, endpoint: str) -> tuple[str, dict]:
    uiref = Identity(unit_name)
    parent_uiref = uiref.parent

    pri_connection = leddar_conf['ro']['connection']
    pub_conf = leddar_conf['rw']['conf']
    pub_conf_aquisition = pub_conf['acquisition']
    pub_conf_loops = pub_conf['object-detectors']

    for leddar_loop in pub_conf_loops:
        leddar_loop['min-score'] = 1

    return unit_name, {
        'bundle': ['libunity_leddar', '1.0.0'],
        'type': parent_uiref.type,
        'name': parent_uiref.ref,
        'units':
        [
            {
                'type': uiref.type,
                'name': uiref.ref,
                'conf':
                {
                    'driver': {
                        'connection': pri_connection,
                        'dist-thr': 5.1,
                        'acquisition': pub_conf_aquisition,
                    },
                    'bg-detector': {
                        'beam-angle': 45.1,
                        'beam-seg-count': 16,
                        'beam-depth-map': [
                            0.1, 0.1, 0.1, 0.1,
                            0.1, 0.1, 0.1, 0.1,
                            0.1, 0.1, 0.1, 0.1,
                            0.1, 0.1, 0.1, 0.1
                        ]
                    },
                    'loop-detector': {
                        'changelog': {
                            'entry-count': 1,
                            'entry-age': 300
                        },
                        'loops': pub_conf_loops
                    },
                    'classifier': {
                        'bike': [80, 220],
                        'car': [220, 560]
                    },
                    'scene-event': {
                        'endpoints': [
                            '/tmp/lg0-tr0-scene'
                        ]
                    },
                    'track-event': {
                        'endpoints': [
                            endpoint
                        ]
                    }
                }
            }
        ]
    }
