from edge import eventlog
from edge.types.exceptions import ParamsError
from edge.types.identity import Identity


def create_unit__speedcam___speedcam(
        unit_name: str,
        location_uiconf: dict,
        intrinsics: str,
        extrinsics: str) -> tuple[str, dict]:

    try:
        pri_jnode = location_uiconf['ro']
        speedcam_jnode: dict = pri_jnode['speedcam']

        limits_jnode = {
            'min-track-size': speedcam_jnode['min-track-size'],
            'avg-plate-u': speedcam_jnode['avg-plate-u'],
            'plate-u': speedcam_jnode.get('plate-u')
        }

    except KeyError:
        eventlog.log_x()
        raise ParamsError()

    uiref = Identity(unit_name)
    parent_uiref = uiref.parent
    return unit_name, {
        'bundle': ['libunity_speedcam', '1.0.0'],
        'type': parent_uiref.type,
        'name': parent_uiref.ref,
        'units':
        [
            {
                'type': uiref.type,
                'name': uiref.ref,
                'conf': {
                    'camera': {
                        'intrinsics': intrinsics,
                        'extrinsics': extrinsics
                    },
                    'limits': limits_jnode
                }
            }
        ]
    }
