from typing import Final

from edge.webserver.sysconf.dgmod.mg.dgconf.unity.mgd_mediagrabber import (
    StatsUnitBuilder, HaltUnitBuilder, StbUnitBuilder, DriverUnitBuilder, VstUnitBuilder)
from edge.webserver.sysconf.dgmod.unity import UnityConfBuilder


class UnitConfBuilder:
    def __init__(self, uimod_id: str, spool_dir: str) -> None:
        self.uimod_id: Final[str] = uimod_id

        self._stats_builder = StatsUnitBuilder('mediagrabber.0~stats.0', spool_dir)
        self._halt_builder = HaltUnitBuilder('mediagrabber.0~halt.0')
        self._driver_builder = DriverUnitBuilder('mediagrabber.0~driver.0')
        self._stb_builder = StbUnitBuilder('mediagrabber.0~stb.0', spool_dir)
        self._vst_builder = VstUnitBuilder('mediagrabber.0~vst.0', spool_dir)

    def setup_stb(self, fifo_name: str) -> None:
        self._stb_builder.setup_endpoint(fifo_name)

    def setup_vst_busstore(
            self, blob_dir: str,
            min_file_count: int,
            max_file_count: int,
            blob_format: str) -> None:

        self._vst_builder.setup_busstore(blob_dir, min_file_count, max_file_count)
        self._vst_builder.setup_bus_format(blob_format)

    def setup_vst_bus(self, args_dirs: list[str], require_mono8: bool = False) -> None:
        self._vst_builder.setup_bus(args_dirs, require_mono8)

    def setup_driver(self, vendor: str, model: str, serial: str) -> None:
        self._driver_builder.setup(vendor, model, serial)

    def setup_driver_cam_vs_acq(self, w: int, h: int, x: int, y: int, fps: int, rotate180: bool) -> None:
        self._driver_builder.setup_cam_vs_acq(w, h, x, y, fps, rotate180)

    def setup_driver_cam_vs_sweep_manual(self, exp_time: int, gain: int, gamma: float) -> None:
        self._driver_builder.setup_cam_vs_sweep_manual(exp_time, gain, gamma)

    def setup_driver_cam_pinbox_ir(self, signal: str, exposure_ratio: int,  pin: str, inverter: bool) -> None:
        self._driver_builder.setup_cam_pinbox_ir(signal, exposure_ratio, pin, inverter)

    def setup_driver_cam_misc(self, data_size: int):
        self._driver_builder.setup_cam_misc(data_size)

    def build(self, conf_dir: str) -> None:
        builder = UnityConfBuilder()
        builder.unit(*self._stats_builder.build())
        builder.unit(*self._halt_builder.build())
        builder.unit(*self._driver_builder.build())

        # TODO: In the case of mv-terminal the mv-camera does not have a link to the radar.
        try:
            builder.unit(*self._stb_builder.build())
        except NotImplementedError:
            pass

        builder.unit(*self._vst_builder.build())
        builder.build(conf_dir)
