from typing import Final

from edge.webserver.sysconf.dgmod.ra.ra_api import ApiConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.ra.uiconf import IUiConfProvider2


class LocationUiConfProvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, uiconf: dict, uiconf_changed: bool) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[dict] = uiconf
        self._uiconf_changed: Final[bool] = uiconf_changed

    def setup_builders(self,
                       api_builder: ApiConfBuilder | None,
                       main_builder: MainConfBuilder,
                       halt_builder: HaltConfBuilder) -> None:

        pri_conf: dict = self._uiconf['ro']
        pub_conf: dict = self._uiconf['rw']

        driver = pri_conf.get('driver') or 'libdriver-smartmicro'
        endpoint = pri_conf.get('uri') or 'ummr0a:///dev/ttyS0'
        main_builder.setup_device(driver, endpoint)
        main_builder.setup_scene_event_zone(pri_conf.get('ra-zone'))

        if api_builder is not None:
            args_jnode: dict = pub_conf['args']

            cam_jnode = args_jnode['camera']
            cam_position_jnode = cam_jnode['position']
            camera_f = cam_position_jnode['forward']
            camera_l = cam_position_jnode['left']
            camera_u = cam_position_jnode['up']

            radar_position_jnode = pri_conf.get('radar', dict()).get('position-offset') or dict()
            radar_f = camera_f + (radar_position_jnode.get('forward') or 0.0)
            radar_l = camera_l + (radar_position_jnode.get('left') or 0.0)
            radar_u = camera_u + (radar_position_jnode.get('up') or 0.0)

            api_builder.setup_unit_position(camera_f, camera_l, camera_u, radar_f, radar_l, radar_u)

            cam_rotation_jnode = cam_jnode['rotation']
            cam_fl = cam_rotation_jnode['azimuth']
            cam_fu = cam_rotation_jnode['elevation']
            cam_lu = cam_rotation_jnode['roll']
            api_builder.setup_camera_orientation(cam_fl, cam_fu, cam_lu)

            radar_rotation_jnode = args_jnode['radar']['rotation']
            radar_fl = radar_rotation_jnode['azimuth']
            radar_fu = radar_rotation_jnode['elevation']
            radar_lu = radar_rotation_jnode['roll']
            api_builder.setup_radar_orientation(radar_fl, radar_fu, radar_lu)

        halt_lines: list[str] | None = pri_conf.get('haltaction')
        if halt_lines:
            halt_builder.setup_lines(halt_lines)
