from typing import Final

from edge.webserver.sysconf.dgmod.ra.ra_api import ApiConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.ra.uiconf import IUiConfProvider2


class MvCameraUiConfPorvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, uiconf: dict, strobe_offset_frames: int) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[dict] = uiconf
        self._offset_frames: Final[int] = strobe_offset_frames

    def setup_builders(self,
                       api_builder: ApiConfBuilder | None,
                       main_builder: MainConfBuilder,
                       halt_builder: HaltConfBuilder) -> None:

        pri_conf: dict = self._uiconf['ro']
        pub_conf: dict = self._uiconf['rw']

        cam_fps = pub_conf['videostream']['fps']
        offset_usec = self._offset_frames * (1e6 / cam_fps)

        main_builder.setup_strobe_react(f'mg{self._uimod_id}-ra{main_builder.uimod_id}-pipe', offset_usec)

        if api_builder is not None:
            intrinsic_jnode: dict = pri_conf.get('intrinsic') or dict()
            frustum_jnode = intrinsic_jnode.get('frustum')

            pixel_size_mm: float = frustum_jnode['pixel-size'] * 0.001
            sensor_wh_px: tuple[int, int] = frustum_jnode['sensor-size']
            sensor_wh_mm: tuple[float, float] = sensor_wh_px[0] * pixel_size_mm, sensor_wh_px[1] * pixel_size_mm
            focal_dist_mm = frustum_jnode['focal-dist']

            api_builder.setup_camera(sensor_wh_px, sensor_wh_mm, focal_dist_mm)
