from typing import Final

from edge.webserver.sysconf.dgmod.rn.rn_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.rn.rn_setupstore import SetupStoreConfBuilder
from edge.webserver.sysconf.dgmod.rn.uiconf import IUiConfProvider2


class _UiConfProvider(IUiConfProvider2):
    def __init__(self, uimod_id: str, uiconf: dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[dict] = uiconf

    def setup_builders(self, main_builder: MainConfBuilder, ss_builder: SetupStoreConfBuilder) -> None:
        pub_jnode: dict = self._uiconf['rw']
        pri_jnode: dict = self._uiconf['ro']
        live_jnode: dict = pub_jnode['live']
        vs_jnode: dict = live_jnode['videostream']
        v_jnode: dict = live_jnode['renderer']['vehicle']

        main_builder.setup_roadway__enabled(v_jnode['video-lane'] != 'never')

        main_builder.setup_service_stats(f'zabbix/or{int(main_builder.uimod_id) % 10}')
        main_builder.setup_image_footer(vs_jnode['frame-w'], live_jnode['footer'])
        main_builder.setup_image_anno__conf(v_jnode)

        markings_jnode: dict = live_jnode.get('markings') or dict()
        with_line_f: bool = markings_jnode.get('line-f', False)
        with_line_l: bool = markings_jnode.get('line-l', False)

        main_builder.setup_image_markings__enabled(with_line_f, with_line_l)

        trafficlight_jnode: list | None = pri_jnode.get('trafficlight')
        if trafficlight_jnode is not None:
            main_builder.setup_image_tl_anno__conf(trafficlight_jnode)

        ss_builder.setup_active(pub_jnode['enabled'])


HttpStreamingUiConfProvider = _UiConfProvider


class RtspStreamingUiConfProvider(_UiConfProvider):
    def __init__(self, uimod_id: str, uiconf: dict) -> None:
        super().__init__(uimod_id, uiconf)

    def setup_builders(self, main_builder: MainConfBuilder, ss_builder: SetupStoreConfBuilder) -> None:
        super().setup_builders(main_builder, ss_builder)

        main_builder.setup_event_blobs('or{0}-or{0}-store'.format(main_builder.uimod_id), 'RGBA')
        main_builder.setup_event_args([
            'or{0}-rs{0}-scene'.format(main_builder.uimod_id)
        ])
