from datetime import datetime

from edge.support import fs
from edge.webserver.sysconf.types.defs import TERM_UICONF_DIR


class TerminalUiconf:
    def __init__(self, uiconf_path: str):
        self._uiconf_path = uiconf_path

    def get_uuid(self) -> str:
        uiconf = fs.load_json3(self._uiconf_path)
        uiconf_pri: dict = uiconf['ro']
        return uiconf_pri['uuid']

    def get_name(self, default: str | None = None) -> str | None:
        uiconf = fs.load_json3(self._uiconf_path, default=None)
        if uiconf:
            uiconf_pub: dict = uiconf.get('rw') or dict()
            return uiconf_pub.get('origin-name', default)

        return default

    def get_place(self, default: str | None = None) -> str | None:
        uiconf = fs.load_json3(self._uiconf_path, default=None)
        if uiconf:
            uiconf_pub: dict = uiconf.get('rw') or dict()
            return uiconf_pub.get('origin-place', default)

        return default

    @property
    def factory_name(self) -> str:
        uiconf = fs.load_json3(self._uiconf_path)
        uiconf_pri: dict = uiconf['ro']
        return uiconf_pri['factory']


class TermAboutUiconf:
    @staticmethod
    def get_conf_path(ui_pid: str) -> str:
        return fs.join_path(TERM_UICONF_DIR, ui_pid, 'about.json')

    @classmethod
    def save_conf(cls, term_pid: str, term_uuid: str,
                  term_ctime: str | None = None,
                  term_place: str | None = None,
                  term_name: str | None = None) -> None:

        conf_path = cls.get_conf_path(term_pid)
        conf = fs.load_json3(conf_path, default={
            'origin': dict()
        })

        conf_origin: dict = conf['origin']
        conf_origin['uuid'] = term_uuid
        conf_origin['place'] = term_place or term_pid
        conf_origin['name'] = term_name or term_pid

        if term_ctime:
            conf_origin['time'] = term_ctime

        elif 'time' not in conf_origin:
            conf_origin['time'] = datetime.now().isoformat()

        fs.write_json_file(conf, conf_path)
