__all__ = [
    'WiegandPcbUiconf',
    'WiegandPcbInputPortUiconf',
    'WiegandPcbRelayPortUiconf'
]

from typing import Self

from edge import eventlog
from edge.webserver.sysconf.uiconf.uiconf2 import UiConf2


class WiegandPcbUiconf(UiConf2):
    def __init__(self, conf: UiConf2):
        super().__init__(conf.raw)

    @classmethod
    def create(cls, factory_conf: dict | None = None) -> Self:
        conf_jnode = {
            'pri': {
                'schema-path': '/opt/edge/share/setupstore/ui/wiegand-pcb-schema.json',
                'lock-path': factory_conf['lock-path'],
                'endpoint': factory_conf['endpoint']
            },
            'pub': {
                'type': 'wiegand-pcb',
                'conf': {
                    'name': 'Wiegand PCB'
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def endpoint(self) -> str:
        return self.pri['endpoint']


class WiegandPcbInputPortUiconf(UiConf2):
    def __init__(self, conf: UiConf2):
        super().__init__(conf.raw)

    @classmethod
    def create(cls, unit_chan: int) -> Self:
        conf_jnode = {
            'pri': {
                'schema-path': '/opt/edge/share/setupstore/ui/wiegand-pcb~input-port-schema.json',
                'address': unit_chan + 2
            },
            'pub': {
                'type': 'wiegand-pcb~input-port',
                'conf': {
                    'name': f'IN{unit_chan + 1}',
                    'active': 'open'
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def ro_bit_address(self) -> int:
        return self.pri['address']

    @property
    def ro_bit_active_state(self) -> bool:
        ui_active = self.pub['conf']['active']
        if ui_active == 'open':
            active = True
        elif ui_active == 'closed':
            active = False
        else:
            eventlog.log_e('fail: unsupported')
            raise NotImplementedError

        return active


class WiegandPcbRelayPortUiconf(UiConf2):
    def __init__(self, conf: UiConf2):
        super().__init__(conf.raw)

    @classmethod
    def create(cls, unit_chan: int) -> Self:
        conf_jnode = {
            'pri': {
                'schema-path': '/opt/edge/share/setupstore/ui/wiegand-pcb~relay-port-schema.json',
                'address': unit_chan
            },
            'pub': {
                'type': 'wiegand-pcb~relay-port',
                'conf': {
                    'name': f'OUT{unit_chan + 1}',
                    'active': 'open'
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def rw_bit_address(self) -> int:
        return self.pri['address']

    @property
    def rw_bit_active_state(self) -> bool:
        ui_active = self.pub['conf']['active']
        if ui_active == 'open':
            active = True
        elif ui_active == 'closed':
            active = False
        else:
            eventlog.log_e('fail: unsupported')
            raise NotImplementedError

        return active
