from typing import Self

from edge import eventlog
from edge.support import fs
from edge.webserver.sysconf.types.conf import IUiConfProvider
from edge.webserver.sysconf.uimod.utype import *


class EsUimod(IUiConfProvider):
    _SCHEMA_PATH = '/opt/edge/share/setupstore/ui/eventstore-ch-schema.json'
    _DEFCONF_PATH = '/opt/edge/share/setupstore/ui/eventstore-ch-jnode.json'
    _CONF_DIR = '/etc/edge/setupstore/admin/actions/eventstore'

    def __init__(self, uimod_ref: str) -> None:
        self._ref = uimod_ref
        self._ch_id = self._ref.split('.')[-1]

    @classmethod
    def create(cls, uimod_ref: str) -> Self:
        return cls(uimod_ref)

    @property
    def ref(self) -> str:
        return self._ref

    @property
    def type(self) -> str:
        return UTYPE_EVENTSTORE

    @property
    def pid(self) -> str:
        eventlog.log_e('fail: unsupported')
        raise NotImplementedError()

    @property
    def id(self) -> str:
        eventlog.log_e('fail: unsupported')
        raise NotImplementedError()

    @property
    def conf_path(self) -> str:
        return fs.join_path(self._CONF_DIR, self._ch_id, '0.json')

    @property
    def lock_path(self) -> str:
        return fs.join_path('/var/run/edge', '{}.lock').format(self.ref)

    @property
    def schema_path(self) -> str:
        return self._SCHEMA_PATH

    def load_conf(self, **kwargs) -> dict:
        """
        :raise FsError:
        """
        return fs.load_json3(self.conf_path, **kwargs)

    def save_conf(self, conf_jnode: dict) -> None:
        fs.write_json_file(conf_jnode, self.conf_path, pretty=True)

    def install(self, name: str, bschan_64k: str) -> dict:
        fs.create_jnode_from_default(self._DEFCONF_PATH, self.conf_path)
        uiconf = self.load_conf()

        uiconf_pub: dict = uiconf['rw']
        uiconf_pub['args']['name'] = name

        uiconf_pri: dict = uiconf['ro']
        uiconf_pri['channel'] = bschan_64k

        self.save_conf(uiconf)
        return uiconf

    def uninstall(self) -> None:
        uiconf_dir = fs.join_path(self._CONF_DIR, self._ch_id)
        fs.silent_rm_r(uiconf_dir)

    def create_backup_dirpath(self) -> str:
        import time
        CTRL_BACKUP_DIR = '/etc/edge/setupstore/backup/'
        backup_action = '{:d}-{}'.format(int(round(time.time() * 1e6)), self.ref)
        backup_dir = fs.join_path(CTRL_BACKUP_DIR, backup_action)
        return backup_dir
