import iso8601

from edge import eventlog
from edge.support import cgi
from edge.webserver.sysconf import ph
from edge.webserver.systest.misc import Services
from edge.webserver.systest.test import Test, TestStep, Summary, TestArgsError


class DatetimeTestStep(TestStep):
    def __init__(self, services: Services, session_ref: str, test_ref: tuple[str, str], step_ref: str):
        super().__init__(services, session_ref, test_ref, step_ref)

    def _edit_log_item_jnode(self, log_item_jnode: dict, body_jnode: dict) -> dict:
        # unpack #/data/report
        report = body_jnode.get('report', False)
        if type(report) != bool:
            eventlog.log_i('fail: http400. param #/report has invalid value')
            raise cgi.CgiError400

        try:
            data_jnode = body_jnode['data']
            time_jnode = data_jnode['time']

            # unpack #/data/time/actual
            if time_jnode.get('actual'):
                eventlog.log_i('fail: http400. param #/data/time/actual is provided')
                raise cgi.CgiError400

            # unpack #/data/time/expected
            expected_time = time_jnode.get('expected')
            if expected_time:
                iso8601.parse_date(expected_time)

            # unpack #/summary
            summary_jnode = Summary.repack(body_jnode.get('summary'))

            # apply changes
            log_item_jnode['report'] = report
            log_item_jnode['data']['time']['expected'] = expected_time
            log_item_jnode['summary'] = summary_jnode
            return log_item_jnode

        except (KeyError, ValueError, iso8601.ParseError) as x:
            eventlog.log_i(f'fail: http400 <{x}>')
            raise cgi.CgiError400


class DatetimeTest(Test):
    def __init__(self, services: Services, session_ref: str, test_ref: tuple[str, str]):
        super().__init__(services, session_ref, test_ref)

    def _mk_step_jnode(self, test_jnode: dict, step_jnode: dict, args_jnode: dict) -> dict:
        if args_jnode:
            eventlog.log_i('fail: http400')
            raise TestArgsError

        term_uuid = test_jnode['ro']['origin']
        term_pid = ph.map_term_uuid_to_pid(term_uuid)
        if term_pid is None:
            raise NotImplementedError

        blob_ts, blob_ref = self._get_live_image(term_pid)
        step_jnode['data'] = {
            'time': {
                'actual': self._services.tz.map_ts_usec_to_isostr(blob_ts)
            },
            'images': {
                'scene': {
                    'ref': blob_ref
                }
            }
        }

        return step_jnode
