import sys
import signal

from time import sleep
from twisted.internet import reactor

from edge import eventlog
from edge.support.argset import ArgSet
from edge.support import fs
from edge.types.exceptions import JnodeError

from renderer._args import decode__kernel_conf
from renderer._kernel import Kernel


# noinspection PyUnusedLocal
def handle_sighup(signal_nubmer: int, interrupted_stack_frame) -> None:
    # traceback.print_stack(interrupted_stack_frame)
    pass


# noinspection PyUnresolvedReferences,PyBroadException
def run_module(raw_arg_set: list[str]) -> None:
    arg_set = ArgSet(raw_arg_set)
    if arg_set.mode == ArgSet.MODE_NONE:
        print("The command line's arguments are invalid")
        sys.exit(1)

    if arg_set.mode == ArgSet.MODE_STARTUP:
        conf_name = f'overlayrender.{arg_set.module_id}'
        eventlog.open_log(conf_name, arg_set.log_level)
        eventlog.log_i('startup...')

        reactor.addSystemEventTrigger('after', 'startup', eventlog.log_i, 'startup... done')
        reactor.addSystemEventTrigger('before', 'shutdown', eventlog.log_i, 'shutdown...')
        signal.signal(signal.SIGHUP, handle_sighup)

        main_path = fs.join_path('/etc/edge', conf_name, 'main.conf')
        try:
            kernel_conf = None
            trottle_wait_args = False
            while kernel_conf is None:
                if trottle_wait_args:
                    sleep(15)

                trottle_wait_args = True

                main_jnode = fs.load_json3(main_path, default=None)
                if main_jnode:
                    try:
                        kernel_conf = decode__kernel_conf(main_jnode, arg_set.log_level)
                    except JnodeError:
                        eventlog.log_e('fail: decode__kernel_conf')

                    except Exception:
                        eventlog.log_x()

            kernel = Kernel(reactor, kernel_conf)
            kernel.run()

        finally:
            eventlog.log_i('shutdown... done')
            eventlog.close_log()


if __name__ == '__main__':
    run_module(sys.argv)
