from abc import ABCMeta, abstractmethod
from typing import Final, List

from edge.services.types.service import IService
from edge.types.image import ImRgba


class ImQueueConf:
    def __init__(self, endpoint: str, min_count: int, max_count: int):
        self.endpoint = endpoint
        self.min_file_count = min_count
        self.max_file_count = max_count


class ImStoreConf:
    def __init__(self, im_format: str, im_q: ImQueueConf):
        self.codec = im_format
        self.image_queue_args = im_q


class JsonQueueConf:
    def __init__(self, endpoint: str, min_count: int, max_count: int):
        self.endpoint = endpoint
        self.min_file_count = min_count
        self.max_file_count = max_count


class JsonStoreConf:
    def __init__(self, q_args: List[JsonQueueConf], log_level: str = 'info'):
        self.json_queue_args_list = q_args
        self.log_level = log_level


class EventEmitterConf:
    def __init__(self, json_store: JsonStoreConf, im_store: ImStoreConf) -> None:
        self.json_store: Final[JsonStoreConf] = json_store
        self.im_store: Final[ImStoreConf] = im_store


class IEventEmitter(IService):
    @abstractmethod
    def emit(self, scene_ts: int, scene_im: ImRgba) -> None:
        pass
