from typing import List, Dict

from edge import eventlog
from edge.types.exceptions import JnodeError
from renderer.services.types.tl import TlLamp, TlConf


def decode__tl_lamp(jnode: List) -> TlLamp:
    """
    :param jnode: #/signal-*
    """
    if len(jnode) != 4:
        eventlog.log_e_param('#/signal-*')
        raise JnodeError

    x0, x1, y0, y1 = jnode
    return TlLamp(x0 / x1, y0 / y1, 0.01)


def decode__tl_conf(jnode: Dict) -> List[TlConf]:
    tl_args_list: List[TlConf] = []
    for tl_jnode in jnode.get('trafficlight', []):
        red = amber = green = None

        red_jnode = tl_jnode.get('signal-r')
        if red_jnode:
            red = decode__tl_lamp(red_jnode)

        amber_jnode = tl_jnode.get('signal-a')
        if amber_jnode:
            amber = decode__tl_lamp(amber_jnode)

        green_jnode = tl_jnode.get('signal-g')
        if green_jnode:
            green = decode__tl_lamp(green_jnode)

        tl_args_list.append(TlConf(red, amber, green))

    return tl_args_list
