from abc import ABCMeta, abstractmethod
from typing import List

from PIL import Image

from edge.image.types.markings import MarkingsConf
from edge.services.types.service import IService
from edge.types.image import ImRgba
from renderer.services.types.tl import TlConf
from renderer.types.tr import TrScene


class LrAnnoConf:
    MODE_NEVER = 'never'
    MODE_ALWAYS = 'always'

    def __init__(self,
                 dir_mode: str = 'never',
                 lane_mode: str = 'never',
                 text_mode: str = 'never',
                 trace_mode: str = 'never',
                 center_mode: str = 'never',
                 vehicle_bbox_mode: str = 'never',
                 vehicle_cuboid_mode: str = 'never'):

        self.show_dir = dir_mode == 'always'
        self.show_lane = lane_mode == 'always'
        self.show_text = text_mode == 'always'
        self.show_trace = trace_mode == 'always'
        self.show_center = center_mode == 'always'
        self.show_vehicle_bbox = vehicle_bbox_mode == 'always'
        self.show_vehicle_cuboid = vehicle_cuboid_mode == 'always'


class RaAnnoConf:
    MODE_IF_PLATE = 'if-plate'
    MODE_ALWAYS = 'always'
    MODE_NEVER = 'never'

    def __init__(self,
                 yellow_speed: int,
                 red_speed: int,
                 len_mode: str = 'never',
                 dir_mode: str = 'never',
                 target_mode: str = 'never',
                 speed_mode: str = 'never',
                 vehicle_mode: str = 'never',
                 radar_present: bool = False):

        self._speed_mode = speed_mode
        self._len_mode = len_mode
        self._dir_mode = dir_mode
        self._target_mode = target_mode
        self._vehicle_mode = vehicle_mode

        self.yellow_speed = yellow_speed
        self.red_speed = red_speed
        self.radar_present = radar_present

    @classmethod
    def _show(cls, mode: str, lr_ra_match: bool) -> bool:
        if mode == cls.MODE_ALWAYS:
            return True

        elif mode == cls.MODE_IF_PLATE:
            return lr_ra_match

        else:
            return False

    def show_len(self, lr_ra_match: bool = False) -> bool:
        return self._show(self._len_mode, lr_ra_match)

    def show_dir(self, lr_ra_match: bool = False) -> bool:
        return self._show(self._dir_mode, lr_ra_match)

    def show_target(self, lr_ra_match: bool = False) -> bool:
        return self._show(self._target_mode, lr_ra_match)

    def show_vehicle(self, lr_ra_match: bool = False) -> bool:
        return self._show(self._vehicle_mode, lr_ra_match)

    def show_speed(self, lr_ra_match: bool = False) -> bool:
        return self._show(self._speed_mode, lr_ra_match)



class RendererConf:
    def __init__(self, lr_anno: LrAnnoConf, ra_anno: RaAnnoConf, tl_args: List[TlConf], lr_store_tout: int, markings: MarkingsConf):
        self.lr_anno = lr_anno
        self.ra_anno = ra_anno
        self.tl_args = tl_args
        self.lr_store_tout = lr_store_tout
        self.markings = markings


class IRenderer(IService):
    @abstractmethod
    def render_tr_scene(self, tr_scene: TrScene, im_rgba: ImRgba) -> Image:
        pass
