from typing import Dict

from edge.types.jnode.imageref import decode__image_ref

from renderer.types.jnode.face import decode__face_scene_item
from renderer.types.jnode.lr import decode__lr_scene_item
from renderer.types.jnode.ra import decode__ra_scene_item
from renderer.types.tr import TrSceneItem, TrScene


def decode__tr_scene_item(jnode: Dict) -> TrSceneItem:
    """
    :param jnode:
        TrScene#/LrRa/:index
    """
    lr_item = decode__lr_scene_item(jnode['Lr'])
    ra_item = decode__ra_scene_item(jnode['Ra'])
    return TrSceneItem(lr_item, ra_item)


def decode__tr_scene(jnode: Dict) -> TrScene:
    """
    :param jnode:
        TrScene#/
    """
    im_ref = decode__image_ref(jnode['Image'])

    tr_items = [decode__tr_scene_item(item_jnode) for item_jnode in jnode.get('LrRa', [])]
    lr_items = [decode__lr_scene_item(item_jnode) for item_jnode in jnode.get('Lr', [])]
    ra_items = [decode__ra_scene_item(item_jnode) for item_jnode in jnode.get('Ra', [])]
    face_items = [decode__face_scene_item(item_jnode) for item_jnode in jnode.get('Face', [])]

    return TrScene(im_ref, tr_items, lr_items, ra_items, face_items)
