from typing import Union, List, Self, Tuple

from edge.types.shape.cuboid import CuboidI


class RaMargin:
    def __init__(self,
                 left: Union[float, int],
                 top: Union[float, int],
                 right: Union[float, int],
                 bottom: Union[float, int]):

        self.left = float(left)
        self.right = float(right)
        self.top = float(top)
        self.bottom = float(bottom)

    @classmethod
    def unpack(cls, jnode: List[float]) -> Self:
        """
        :param jnode:
            TrScene#/Ra-ltrb
        """
        if len(jnode) != 4:
            raise ValueError

        return cls(*jnode)


class RaSceneItem:
    def __init__(self, rel_im_x: float, rel_im_y: float, ra_speed: int, ra_len: int, ra_cuboid: CuboidI,
                 ra_f_cm: int, ra_l_cm: int):
        self.ra_rel_im_x = rel_im_x
        self.ra_rel_im_y = rel_im_y

        self.ra_speed = ra_speed
        self.ra_len = ra_len
        self.ra_cuboid = ra_cuboid

        self.ra_f_cm = ra_f_cm
        self.ra_l_cm = ra_l_cm

    @property
    def ra_dir(self) -> int:
        if self.ra_speed > 0:
            return 12
        elif self.ra_speed < 0:
            return 11
        else:
            return 1

    def map_bbox_ref(self, target_im_w: int, target_im_h: int) -> Tuple[int, int]:
        abs_x = self.ra_rel_im_x * target_im_w
        abs_y = self.ra_rel_im_y * target_im_h
        return int(abs_x), int(abs_y)
