from typing import Final, Optional, Self

from edge.services.types.cam import ICamService
from edge.services.types.about import IAboutService
from edge.services.types.tz import ITzService
from edge.support.service import ServicePack

from edge.services.types.vs import IVsService


class RuleDeps:
    def __init__(self,
                 tz_service: ITzService,
                 vs_service: IVsService,
                 about_service: IAboutService,
                 cam_service: Optional[ICamService]):

        self.tz: Final[ITzService] = tz_service
        self.vs: Final[IVsService] = vs_service
        self.about: Final[IAboutService] = about_service
        self.cam: Final[ICamService | None] = cam_service

    @classmethod
    def create(cls, services: ServicePack) -> Self:
        return cls(
            services.get(ITzService),
            services.get(IVsService),
            services.get(IAboutService),
            services.try_get(ICamService))


class DetectorDeps(RuleDeps):
    pass
