from typing import Dict

from edge.types.identity import Identity
from speedcontrol.detectors.spotspeed_waga.types.stats import DetectorStats, RuleStats


def encode__rule_stats(name: str, stats: RuleStats) -> Dict:
    make_evidence_total_fail = stats.evidence_failed_make_images + stats.evidence_failed_load_images
    total_fail = (stats.schedule_unmatch +
                  stats.lane_unmatch +
                  stats.raspeed_valid +
                  stats.class_unmatch)

    return {
        'type': 'spot-speed-waga~rule',
        'name': name,
        'stat': {
            'assert-track': {
                'count': [
                    stats.total,
                    stats.total - total_fail,
                    total_fail
                ],
                'type-unmatch': stats.class_unmatch,
                'schedule-unmatch': stats.schedule_unmatch,
                'lane-unmatch': stats.lane_unmatch,
                'ra-speed-valid': stats.raspeed_valid
            },
            'make-evidence': {
                'count': [
                    stats.make_evidence_total,
                    stats.make_evidence_total - make_evidence_total_fail,
                    make_evidence_total_fail
                ],
                'load-images': stats.evidence_failed_load_images,
                'make-images': stats.evidence_failed_make_images
            }
        }
    }


def encode__detector_stats(idenity: Identity, stats: DetectorStats) -> dict:
    total_fails = stats.lr_absent + stats.lr_invalid + stats.wa_absent + stats.ra_cam_speed
    return {
        'type': 'spot-speed-waga',
        'name': f'{idenity.ref}',
        'stat': {
            'assert-track': {
                'count': [stats.total, stats.total - total_fails, total_fails],
                'wa-absent': stats.wa_absent,
                'lr-absent': stats.lr_absent,
                'lr-invalid': stats.lr_invalid,
                'racam-speed-invalid': stats.ra_cam_speed
            },
            'emit-event': {
                'count': [
                    stats.emit_total,
                    stats.emit_total - stats.emit_fail,
                    stats.emit_fail
                ]
            }
        }
    }
