from typing import Dict, Self, Optional

from edge import eventlog
from edge.types.exceptions import JnodeError

from speedcontrol.detectors.types.jnode.conf import decode__schedule_conf
from speedcontrol.detectors.transit.types.conf import DetectorConf, ImagesArgs
from edge.types.identity import Identity


class JsonImagesArgs(ImagesArgs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def deserialize(cls, args_jnode: Dict) -> Self:
        """
        :raise JnodeError:
        """
        def deserialize_store(im_profile):
            profile_jnode = args_jnode.get(im_profile)
            if dict != type(profile_jnode):
                eventlog.log_e_param('#%s' % im_profile)
                raise JnodeError

            store = profile_jnode.get('store')
            if not (type(store) in [str] and 1 <= len(store) <= 8):
                eventlog.log_e_param('#/%s/store' % im_profile)
                raise JnodeError

            return store

        scene_store, vehicle_store = list(map(
            deserialize_store, ['scene', 'vehicle']))

        return cls(vehicle_store, scene_store)


def decode__detector_conf(identity: Identity, jnode: Dict, def_footer_mapping: Optional[Dict] = None) -> DetectorConf:
    """
    :param identity:
    :param jnode:
        /etc/edge/speedcontrol.x/transit.json#/
    :param def_footer_mapping:
    """
    rules_jnode = jnode['rules']

    # deserialize #/rules
    if type(rules_jnode) != list or len(rules_jnode) != 1:
        eventlog.log_e_param('#/rules')
        raise JnodeError

    # deserialize #/rules[0]/schedule
    schedule_args = None
    schedule_jnode = rules_jnode[0].get('schedule')
    if schedule_jnode is not None:
        if dict != type(schedule_jnode):
            eventlog.log_e_param('#/schedule')
            raise JnodeError

        schedule_args = decode__schedule_conf(schedule_jnode)

    # deserialize #/rules[0]/name
    name = rules_jnode[0].get('name')
    if type(name) not in [str]:
        eventlog.log_e_param('#/name')
        raise JnodeError

    # deserialize #/rules[0]/enabled
    enabled = rules_jnode[0].get('enabled')
    if type(enabled) != bool:
        eventlog.log_e_param('#/enabled')
        raise JnodeError

    # deserialize #/rules[0]/limits
    limits_jnode = rules_jnode[0].get('limits')
    if type(limits_jnode) != dict:
        eventlog.log_e_param('#/limits')
        raise JnodeError

    require_plate = limits_jnode.get('require-plate', True)
    if type(require_plate) != bool:
        eventlog.log_e_param('#/limits/require-plate')
        raise JnodeError

    # deserialize #/images
    images_jnode = jnode.get('images')
    if type(images_jnode) != dict:
        eventlog.log_e_param('#/images')
        raise JnodeError

    # deserialize #/images/best
    best_waypoint_jnode = images_jnode.get('best')
    if type(best_waypoint_jnode) != dict:
        eventlog.log_e_param('#/images/best')
        raise JnodeError

    images = JsonImagesArgs.deserialize(best_waypoint_jnode)

    plate_detected_mode = jnode.get('plate-detected-mode', False)
    le_loop_ref = jnode.get('le-loop-ref', None)
    with_video = jnode.get('with-video', False)
    return DetectorConf(name, enabled, require_plate, schedule_args, images, identity,
                        plate_detected_mode, le_loop_ref, with_video)
