from typing import Dict

from edge.services.types.tz import ITzService
from edge.types.origin import Origin

from speedcontrol.events.detector import DetectorInfo
from speedcontrol.events.evidence import BaseEvidence
from speedcontrol.events.jnode.schedule import encode__schedule_info


class Evidence(BaseEvidence):
    def __init__(self, origin: Origin, event_ts: int, detector_info: DetectorInfo, standing_time_sec: int):
        self.detector = detector_info
        self._standing_time = standing_time_sec

        super().__init__(BaseEvidence.EVENT_TYPE__PARKING, origin, event_ts)

    @property
    def standing_time(self) -> int:
        return self._standing_time

    def serialize(self, tz_service: ITzService) -> Dict:
        jnode = BaseEvidence.serialize(self, tz_service)
        waypoints_jnode = [x.serialize(tz_service) for x in self._waypoints]
        jnode['evidence'] = {
            'detector': {
                'name': self.detector.name,
                'rule': {
                    'name': self.detector.rule.name,
                    'schedule': encode__schedule_info(self.detector.rule.schedule),
                    'ticket': {
                        'name': self.detector.rule.ticket.name
                    },
                    'limits': self.detector.rule.limits
                }
            },
            'measurements': {
                'standing-time': self._standing_time,
                'waypoints': {
                    'violation': waypoints_jnode
                }
            }
        }

        return jnode
