from typing import Dict, cast

from edge.services.types.tz import ITzService
from edge.types.origin import Origin
from speedcontrol.events.detector import DetectorInfo
from speedcontrol.events.evidence import BaseEvidence
from speedcontrol.events.waypoint.redlight import RedLightViolationWaypoint
from edge.services.types.tl import TlHaltSpanInfo


class Evidence(BaseEvidence):
    def __init__(self, origin: Origin, ts: int, detector_info: DetectorInfo):
        super().__init__(BaseEvidence.EVENT_TYPE__REDLIGHT, origin, ts)
        self.detector = detector_info

    def serialize(self, tz_service: ITzService) -> Dict:
        jnode = BaseEvidence.serialize(self, tz_service)
        jnode['evidence'] = {

        }
        # waypoints_jnode = dict()
        # for waypoint in self.waypoints:
        #     waypoints_jnode[waypoint.name] = waypoint.serialize(tz_service)

        pre_waypoints = [waypoint for waypoint in self._waypoints
                         if waypoint.name == waypoint.NAME_PRE_STOPBAR]
        post_waypoints = [waypoint for waypoint in self._waypoints
                          if waypoint.name == waypoint.NAME_POST_STOPBAR]

        pre_waypoints_jnode = [x.serialize(tz_service) for x in pre_waypoints]
        post_waypoints_jnode = [x.serialize(tz_service) for x in post_waypoints]

        jnode['evidence'] = {
            'detector': {
                'name': self.detector.name,
                'rule': {
                    'name': self.detector.rule.name,
                    'ticket': {
                        'name': self.detector.rule.ticket.name
                    }
                }
            },
            'measurements': {
                'waypoints': {
                    'pre-stopbar': pre_waypoints_jnode,
                    'post-stopbar': post_waypoints_jnode
                }
            }
        }

        if len(self._videos):
            videos_jnode = [x.serialize(tz_service) for x in self._videos]
            jnode['evidence']['measurements']['videos'] = videos_jnode

        return jnode

    def get_ukr_json_ev(self) -> TlHaltSpanInfo:
        """
        :returns
            (green_amber_time, red_time, red_amber_time, halt_time)
        """
        post_waypoints = [x for x in self._waypoints if x.name == x.NAME_POST_STOPBAR]
        post_waypoints.sort(key=lambda item: item.main_ts, reverse=True)

        post_waypoint_last = cast(RedLightViolationWaypoint, post_waypoints[0])
        return post_waypoint_last.halt_span_info
