from typing import List, Dict

from edge.services import db_wanted
from edge.services.types.tz import ITzService
from edge.types.blob import MemBlob
from edge.types.origin import Origin

from speedcontrol.events.detector import DetectorInfo
from speedcontrol.events.evidence import BaseEvidence


class Evidence(BaseEvidence):

    def __init__(self,
                 origin: Origin,
                 event_ts: int,
                 detector_info: DetectorInfo,
                 matches: List[db_wanted.ListMatch]):

        super().__init__(BaseEvidence.EVENT_TYPE__WANTED, origin, event_ts)
        self.detector = detector_info
        self.matches = matches

    def get_blobs(self) -> List[MemBlob]:
        return []

    def serialize(self, tz_service: ITzService) -> Dict:
        jnode = BaseEvidence.serialize(self, tz_service)
        jnode['evidence'] = {
            'detector': {
                'name': self.detector.name,
                'rule': {
                    'name': self.detector.rule.name,
                    'ticket': {
                        'name': self.detector.rule.ticket.name
                    },
                    'limits': self.detector.rule.limits
                }
            },
            'measurements': {
                'matches': [x.serialize() for x in self.matches]
            }
        }

        return jnode
