from typing import Dict, Final, List, Optional

from edge.services.types.tz import ITzService
from edge.types.blob import MemBlob
from edge.types.location import Location

from speedcontrol.events.jnode.vehicle import encode__violation_vehicle
from speedcontrol.events.vehicle import ViolationVehicle
from speedcontrol.events.waypoint import Waypoint
from edge.services.types.tl import TlHaltSpanInfo


class RedLightViolationWaypoint(Waypoint):
    def __init__(self,
                 name: str,
                 main_ts: int,
                 location: Location,
                 images: List[MemBlob],
                 vehicle: Optional[ViolationVehicle],
                 halt_span_info: TlHaltSpanInfo):

        super().__init__(name, main_ts, None, location, images)

        self.vehicle: Final[ViolationVehicle] = vehicle
        self.halt_span_info: Final[TlHaltSpanInfo] = halt_span_info

    def serialize(self, tz_service: ITzService, with_images: bool = True) -> Dict:
        jnode = Waypoint.serialize(self, tz_service, with_images)

        vehicle_jnode: Optional[Dict] = encode__violation_vehicle(self.vehicle)
        if vehicle_jnode is not None:
            jnode.update({
                'vehicle': vehicle_jnode,
            })

        if self.halt_span_info.green_amber_time is not None:
            jnode.update({'green-amber-time': self.halt_span_info.green_amber_time})

        if self.halt_span_info.red_time is not None:
            jnode.update({'red-time': self.halt_span_info.red_time})

        if self.halt_span_info.red_amber_time is not None:
            jnode.update({'red-amber-time': self.halt_span_info.red_amber_time})

        if self.halt_span_info.halt_time is not None:
            jnode.update({'halt-time': self.halt_span_info.halt_time})

        return jnode
