from tracker.services.types.ra_track_provider import RaTrackProviderConf, RaTrackFilterRuleConf, RaTrackFilterConf, \
    RaTrackProviderStats


def encode__ra_track_provider_stats(stats: RaTrackProviderStats) -> dict:
    pop_track_errors = (stats.pop_track.failure_not_found_count
                        + stats.pop_track.failure_too_short_count
                        + stats.pop_track.failure_mmr_speed_limit_count)

    del_track_errors = stats.del_track.failure_too_short_count

    jnode = {
        'type': 'tracker~ra-track-provider',
        'name': 'tracker.0~ra-track-provider.0',
        'stat': {
            'add-track': {
                'count': stats.add_track.count
            },
            'pop-track': {
                'count': [
                    stats.pop_track.success_count + pop_track_errors,
                    stats.pop_track.success_count,
                    pop_track_errors
                ],
                'errors': {
                    'not-found': stats.pop_track.failure_not_found_count,
                    'too-short': stats.pop_track.failure_too_short_count,
                    'mmr-speed-limit': stats.pop_track.failure_mmr_speed_limit_count
                }
            },
            'del-track': {
                'count': [
                    stats.del_track.success_count + stats.del_track.failure_too_short_count,
                    stats.del_track.success_count,
                    stats.del_track.failure_too_short_count
                ],
                'errors': {
                    'too-short': stats.del_track.failure_too_short_count
                }
            }
        }
    }
    return jnode


def decode__ra_track_provider_conf(jnode: dict) -> RaTrackProviderConf:
    require_plate = jnode.get('require-plate', True)
    filters_jnode = jnode['filter']
    active_rules: list[RaTrackFilterRuleConf] = []
    for filter_jnode in filters_jnode:
        if not filter_jnode.get('active', True):
            continue

        max_speed = filter_jnode['max-speed']
        mmr_classes = set(filter_jnode['mmr-class'])
        active_rules.append(RaTrackFilterRuleConf(mmr_classes, max_speed))

    return RaTrackProviderConf(require_plate, RaTrackFilterConf(active_rules))
