from typing import Dict, Optional

from edge.types.jnode.shape.cuboid import encode__cuboidi
from edge.types.shape.cuboid import CuboidI
from tracker.types.ra import RaSceneItem, RaScene, RaTrack


def encode__ra_scene_item(self: RaSceneItem, ra_cuboid: CuboidI) -> Dict:
    return {
        'oid': self.oid,
        'len': self.len_cm,
        'speed': self.speed_kmh,
        'xy': [self.frame_l_rel, self.frame_t_rel],
        'c': encode__cuboidi(ra_cuboid),
        'f': self.scene_f_cm,
        'l': self.scene_l_cm
    }


def decode__ra_scene_item(jnode: Dict, frame_ts: int, scene_ts: int) -> Optional[RaSceneItem]:
    """
    :param jnode:
        RaScene#/scene/<index>
    :param frame_ts
    :param scene_ts:
    """
    frame_l_rel, frame_t_rel = jnode['img-lt']
    if not (0.0 < frame_l_rel < 1.0 and 0.0 < frame_t_rel < 1.0):
        return

    oid = jnode['oid']
    oid_stat = int(jnode['stat'])

    scene_f_cm: int = jnode.get('f', 0)
    scene_l_cm: int = jnode.get('l', 0)

    # speed_kmh is always positive
    speed_kmh = int(jnode['kmh'])
    len_cm = int(jnode['len'])

    return RaSceneItem(oid, oid_stat,
                       frame_ts, float(frame_l_rel), float(frame_t_rel),
                       scene_ts, scene_f_cm, scene_l_cm,
                       speed_kmh, len_cm)


def encode__ra_track_item(self: RaSceneItem) -> Dict:
    return {
        'Ts': self.frame_ts,
        'scene-ts': self.scene_ts,
        'Xy': [self.frame_l_rel, self.frame_t_rel],
        'S': self.speed_kmh,
        'F': self.scene_f_cm,
        'L': self.scene_l_cm
    }


def decode__ra_scene(jnode: Dict) -> RaScene:
    """
    :param jnode: RaScene#/
    :raise ValueError:
    """
    if not jnode:
        raise ValueError

    status = int(jnode['stat'])

    if status == RaScene.STATUS_VALID:
        strobe_ts = int(jnode['strobe-ts'])
        scene_ts = int(jnode['scene-ts'])
        all_scene_items = [
            decode__ra_scene_item(item_jnode, strobe_ts, scene_ts)
            for item_jnode in jnode['scene']
        ]

        ra_items = [x for x in all_scene_items if x]

    else:
        ra_items = []

    return RaScene(ra_items, status)


def encode_ra_track(self: RaTrack) -> Dict:
    return {
        'Length': self.max_length,
        'Trace': [encode__ra_track_item(x) for x in self.trace],
        'Speed2': self.speed_2_kmh,
        'SpeedN': self.speed_n_kmh
    }
